"""
Classes pour les différents types de données de l'API Atmo Data
"""

from typing import List, Dict, Any, Optional, Tuple
from datetime import datetime
import statistics
from dataclasses import dataclass

from .constants import (
    AASQA_CODES, INDICES_ATMO, INDICES_POLLENS, CODE_COLOR_QUALIF,
    CODE_POLLUANT, CODE_TAXON, SECTEURS_EMISSIONS, CODE_POLLUANT_EPISODES, TAXON_MAPPING
)


@dataclass
class Coordinates:
    """Classe pour représenter les coordonnées géographiques"""
    longitude: float
    latitude: float
    
    def __str__(self) -> str:
        return f"({self.latitude:.4f}, {self.longitude:.4f})"
    
    def distance_to(self, other: 'Coordinates') -> float:
        """Calcule la distance approximative entre deux points (en km)"""
        import math
        
        lat1, lon1 = math.radians(self.latitude), math.radians(self.longitude)
        lat2, lon2 = math.radians(other.latitude), math.radians(other.longitude)
        
        dlat = lat2 - lat1
        dlon = lon2 - lon1
        
        a = math.sin(dlat/2)**2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon/2)**2
        c = 2 * math.asin(math.sqrt(a))
        r = 6371  # Rayon de la Terre en km
        
        return r * c


class AtmoDataBase:
    """Classe de base pour tous les types de données Atmo"""
    
    def __init__(self, feature: Dict[str, Any]):
        self.raw_data = feature
        self.properties = feature.get('properties', {})
        self.geometry = feature.get('geometry', {})
        
        # Propriétés communes
        self.aasqa = self.properties.get('aasqa', '')
        self.source = self.properties.get('source', '')
        self.date_maj = self.properties.get('date_maj', '')
        self.lib_zone = self.properties.get('lib_zone', '')
        self.coordinates = self._extract_coordinates()
    
    def _extract_coordinates(self) -> Optional[Coordinates]:
        """Extrait les coordonnées du feature"""
        if self.geometry and self.geometry.get('type') == 'Point':
            coords = self.geometry.get('coordinates', [])
            if len(coords) >= 2:
                return Coordinates(coords[0], coords[1])
        elif 'x_wgs84' in self.properties and 'y_wgs84' in self.properties:
            return Coordinates(
                self.properties['x_wgs84'],
                self.properties['y_wgs84']
            )
        return None
    
    def get_aasqa_name(self) -> str:
        """Retourne le nom de l'AASQA"""
        aasqa_info = AASQA_CODES.get(self.aasqa)
        if aasqa_info:
            return aasqa_info['description']
        return f"AASQA {self.aasqa}"
    
    def get_aasqa_website(self) -> str:
        """Retourne le site web de l'AASQA"""
        aasqa_info = AASQA_CODES.get(self.aasqa)
        if aasqa_info:
            return aasqa_info['site_web']
        return None
    
    def get_aasqa_region(self) -> str:
        """Retourne la région de l'AASQA"""
        aasqa_info = AASQA_CODES.get(self.aasqa)
        if aasqa_info:
            return aasqa_info['region']
        return None
    
    def get_aasqa_organisme(self) -> str:
        """Retourne l'organisme de l'AASQA"""
        aasqa_info = AASQA_CODES.get(self.aasqa)
        if aasqa_info:
            return aasqa_info['organisme']
        return None
    
    def get_source(self) -> str:
        """Retourne le nom publique de l'AASQA, texte"""
        return self.source
    
    def has_coordinates(self) -> bool:
        """Vérifie si l'objet a des coordonnées"""
        return self.coordinates is not None
    
    def get_emoji_by_level(self, level: int, style: str = "round") -> str:
        """Récupère l'émoji correspondant au niveau de qualificatif
        
        Args:
            level: Niveau de qualificatif (0-7)
            style: Style d'émoji - "round" pour ronds (🟢), "square" pour carrés (🟩)
        
        Returns:
            str: Émoji correspondant au niveau et style demandé
        """
        color_info = CODE_COLOR_QUALIF.get(level, CODE_COLOR_QUALIF.get(0, [None, None, "❓", "❓"]))
        
        if style == "square" and len(color_info) > 3:
            # Utiliser l'émoji carré (4ème élément)
            return color_info[3]
        elif len(color_info) > 2:
            # Utiliser l'émoji rond (3ème élément)
            return color_info[2]
        else:
            # Fallback
            return "❓"

    def get_color_by_level(self, level: int) -> Tuple[str, List[int]]:
        """Récupère la couleur (hex, rgb) correspondant au niveau de qualificatif"""
        color_info = CODE_COLOR_QUALIF.get(level)
        if color_info and len(color_info) >= 2:
            return color_info[1], color_info[0]  # hex, rgb
        return "#DDDDDD", [221, 221, 221]  # Couleur par défaut

class IndiceAtmo(AtmoDataBase):
    """Classe pour les indices de qualité de l'air ATMO"""
    
    def __init__(self, feature: Dict[str, Any]):
        super().__init__(feature)
        
        # Propriétés spécifiques aux indices ATMO
        self.code_qual = self.properties.get('code_qual', 0)
        self.lib_qual = self.properties.get('lib_qual', '')
        self.coul_qual = self.properties.get('coul_qual', '')
        self.code_zone = self.properties.get('code_zone', '')
        self.lib_zone = self.properties.get('lib_zone', '')
        self.date_dif = self.properties.get('date_dif', '')
        self.date_ech = self.properties.get('date_ech', '')
        
        # Champs supplémentaires selon la notice officielle
        self.type_zone = self.properties.get('type_zone', '')  # 'commune' ou 'EPCI'
        self.x_reg = self.properties.get('x_reg', 0.0)  # Coordonnées réglementaires
        self.y_reg = self.properties.get('y_reg', 0.0)
        self.epsg_reg = self.properties.get('epsg_reg', '')  # Système de projection réglementaire
        
        # Concentrations facultatives (selon notice)
        self.conc_no2 = self.properties.get('conc_no2', 0)  # Concentration en μg/m³
        self.conc_so2 = self.properties.get('conc_so2', 0)
        self.conc_o3 = self.properties.get('conc_o3', 0)
        self.conc_pm10 = self.properties.get('conc_pm10', 0)
        self.conc_pm25 = self.properties.get('conc_pm25', 0)
        
        # Codes par polluant
        self.code_no2 = self.properties.get('code_no2', 0)
        self.code_so2 = self.properties.get('code_so2', 0)
        self.code_o3 = self.properties.get('code_o3', 0)
        self.code_pm10 = self.properties.get('code_pm10', 0)
        self.code_pm25 = self.properties.get('code_pm25', 0)
    
    def get_qualificatif(self) -> str:
        """Retourne le qualificatif textuel de l'indice"""
        return INDICES_ATMO.get(self.code_qual, "Inconnu")
    
    def get_color(self) -> Tuple[str, List[int]]:
        """Retourne la couleur (hex, rgb) associée à l'indice"""
        return self.get_color_by_level(self.code_qual)
    
    def get_emoji(self, style: str = "round") -> str:
        """Retourne l'émoji correspondant à l'indice ATMO
        
        Args:
            style: Style d'émoji - "round" pour ronds (🟢), "square" pour carrés (🟩)
        """
        return self.get_emoji_by_level(self.code_qual, style)
    
    def is_good_quality(self) -> bool:
        """Vérifie si la qualité de l'air est bonne (indice 1-2)"""
        return self.code_qual in [1, 2]
    
    def is_poor_quality(self) -> bool:
        """Vérifie si la qualité de l'air est mauvaise (indice 4+)"""
        return self.code_qual >= 4
    
    def get_worst_pollutant(self) -> Tuple[str, int]:
        """Retourne le polluant avec le plus mauvais indice"""
        pollutants = {
            'NO2': self.code_no2,
            'SO2': self.code_so2,
            'O3': self.code_o3,
            'PM10': self.code_pm10,
            'PM2.5': self.code_pm25
        }
        worst = max(pollutants.items(), key=lambda x: x[1])
        return worst[0], worst[1]
    
    def get_pollutants_summary(self) -> Dict[str, Dict[str, Any]]:
        """Retourne un résumé de tous les polluants"""
        pollutants = {
            'NO2': self.code_no2,
            'SO2': self.code_so2,
            'O3': self.code_o3,
            'PM10': self.code_pm10,
            'PM2.5': self.code_pm25
        }
        
        summary = {}
        for polluant, code in pollutants.items():
            summary[polluant] = {
                'code': code,
                'qualificatif': INDICES_ATMO.get(code, "Inconnu"),
                'description': CODE_POLLUANT.get(polluant, polluant)
            }
        
        return summary
    
    def get_concentrations(self) -> Dict[str, int]:
        """Retourne les concentrations de tous les polluants en μg/m³ (selon notice officielle)"""
        return {
            'NO2': self.conc_no2,
            'SO2': self.conc_so2,
            'O3': self.conc_o3,
            'PM10': self.conc_pm10,
            'PM2.5': self.conc_pm25
        }
    
    def is_commune_level(self) -> bool:
        """Vérifie si l'indice est calculé au niveau commune (selon notice officielle)"""
        return self.type_zone.lower() == 'commune'
    
    def is_epci_level(self) -> bool:
        """Vérifie si l'indice est calculé au niveau EPCI (selon notice officielle)"""
        return self.type_zone.lower() == 'epci'
    
    def get_responsible_pollutants(self) -> List[str]:
        """Retourne le(s) polluant(s) responsable(s) de l'indice global (selon règle n°4 de la notice)"""
        pollutants = {
            'NO2': self.code_no2,
            'SO2': self.code_so2,
            'O3': self.code_o3,
            'PM10': self.code_pm10,
            'PM2.5': self.code_pm25
        }
        
        # L'indice global correspond au qualificatif le plus dégradé
        max_code = max(pollutants.values())
        responsible = [polluant for polluant, code in pollutants.items() if code == max_code]
        
        return responsible
    
    def __str__(self) -> str:
        return f"Indice ATMO {self.lib_zone}: {self.get_qualificatif()} ({self.code_qual})"


class EpisodePollution(AtmoDataBase):
    """Classe pour les épisodes de pollution"""
    
    def __init__(self, feature: Dict[str, Any]):
        super().__init__(feature)
        
        # Propriétés spécifiques aux épisodes
        self.code_pol = self.properties.get('code_pol', '')
        self.lib_pol = self.properties.get('lib_pol', '')
        self.code_zone = self.properties.get('code_zone', '')
        self.lib_zone = self.properties.get('lib_zone', '')
        self.date_dif = self.properties.get('date_dif', '')
        self.date_ech = self.properties.get('date_ech', '')
        self.etat = self.properties.get('etat', '')
    
    def get_polluant_code(self) -> str:
        """Retourne le code du polluant principal"""
        return CODE_POLLUANT_EPISODES.get(self.code_pol, self.code_pol)
    
    def is_alert_active(self) -> bool:
        """Vérifie si une alerte est active"""
        return self.etat not in ['PAS DE DEPASSEMENT', '']
    
    def get_alert_level(self) -> str:
        """Retourne le niveau d'alerte"""
        if 'INFORMATION' in self.etat:
            return 'Information'
        elif 'ALERTE' in self.etat:
            return 'Alerte'
        else:
            return 'Aucune'
    
    def is_geometry_complex(self) -> bool:
        """Vérifie si la géométrie est complexe (MultiPolygon)"""
        return self.geometry.get('type') == 'MultiPolygon'
    
    def __str__(self) -> str:
        return f"Épisode {self.lib_pol} - {self.lib_zone}: {self.etat}"


class EmissionData(AtmoDataBase):
    """Classe pour les données d'émissions"""
    
    def __init__(self, feature: Dict[str, Any]):
        super().__init__(feature)
        
        # Propriétés spécifiques aux émissions
        self.code = self.properties.get('code', '')
        self.name = self.properties.get('name', '')
        self.population = self.properties.get('population', 0)
        self.superficie = self.properties.get('superficie', 0)
        
        # Émissions par polluant
        self.nox = self.properties.get('nox', 0)
        self.pm10 = self.properties.get('pm10', 0)
        self.pm25 = self.properties.get('pm25', 0)
        self.ges = self.properties.get('ges', 0)  # Gaz à effet de serre
        
        # Secteur
        self.code_pcaet = self.properties.get('code_pcaet', '')
    
    def get_emission_density(self, pollutant: str) -> float:
        """Calcule la densité d'émission par km² pour un polluant"""
        emission_value = getattr(self, pollutant.lower(), 0)
        if self.superficie > 0:
            return emission_value / self.superficie
        return 0
    
    def get_emission_per_capita(self, pollutant: str) -> float:
        """Calcule l'émission par habitant pour un polluant"""
        emission_value = getattr(self, pollutant.lower(), 0)
        if self.population > 0:
            return emission_value / self.population
        return 0
    
    def get_total_emissions(self) -> Dict[str, float]:
        """Retourne toutes les émissions"""
        return {
            'NOx': self.nox,
            'PM10': self.pm10,
            'PM2.5': self.pm25,
            'GES': self.ges
        }
    
    def get_secteur_name(self) -> str:
        """Retourne le nom du secteur d'émission"""
        return SECTEURS_EMISSIONS.get(self.code_pcaet, f"Secteur {self.code_pcaet}")
    
    def __str__(self) -> str:
        return f"Émissions {self.name}: NOx={self.nox:.1f}, PM10={self.pm10:.1f}"


class IndicePollen(AtmoDataBase):
    """Classe pour les indices pollen"""
    
    def __init__(self, feature: Dict[str, Any]):
        super().__init__(feature)
        
        # Propriétés spécifiques aux pollens
        # Notice : Déclenchement ou non de l’alerte pollen (à partir de code_qual = 4) -> Boolean
        self.alerte = self.properties.get('alerte', False)
        
        # Codes par taxon (espèce)
        # Notice : Classe du sous-indice, entier de 1 à 6, 0 si absent -> Int
        self.code_ambr = self.properties.get('code_ambr', 0)  # Ambroisie
        self.code_arm = self.properties.get('code_arm', 0)   # Armoise
        self.code_aul = self.properties.get('code_aul', 0)   # Aulne
        self.code_boul = self.properties.get('code_boul', 0) # Bouleau
        self.code_gram = self.properties.get('code_gram', 0) # Graminées
        self.code_oliv = self.properties.get('code_oliv', 0) # Olivier
        
        # Concentrations par taxon (grains/m³)
        # Notice : Concentration de pollens du taxons (en grains/m³), à la commune -> Float
        self.conc_ambr = self.properties.get('conc_ambr', 0.0)  # Ambroisie
        self.conc_arm = self.properties.get('conc_arm', 0.0)   # Armoise
        self.conc_aul = self.properties.get('conc_aul', 0.0)   # Aulne
        self.conc_boul = self.properties.get('conc_boul', 0.0) # Bouleau
        self.conc_gram = self.properties.get('conc_gram', 0.0) # Graminées
        self.conc_oliv = self.properties.get('conc_oliv', 0.0) # Olivier
        
        # Taxon(s) responsable(s) de l'indice
        # Notice : Taxon(s) responsable(s) de l'indice (aulne, bouleau, olivier, graminées, armoise, ambroisie) -> String
        self.pollen_resp = self.properties.get('pollen_resp', '')
    
    def is_alert_active(self) -> bool:
        """Vérifie si une alerte pollen est active"""
        return self.alerte
    
    def get_highest_pollen(self) -> Tuple[str, int]:
        """Retourne le pollen avec l'indice le plus élevé"""
        pollens = {
            'ambr': self.code_ambr,
            'arm': self.code_arm,
            'aul': self.code_aul,
            'boul': self.code_boul,
            'gram': self.code_gram,
            'oliv': self.code_oliv
        }
        highest = max(pollens.items(), key=lambda x: x[1])
        return highest[0], highest[1]
    
    def get_pollens_summary(self, emoji_style: str = "round") -> Dict[str, Dict[str, Any]]:
        """Retourne un résumé de tous les pollens
        
        Args:
            emoji_style: Style d'émoji - "round" pour ronds (🟢), "square" pour carrés (🟩)
        """
        pollens = {
            'ambr': {'code': self.code_ambr, 'conc': self.conc_ambr},
            'arm': {'code': self.code_arm, 'conc': self.conc_arm},
            'aul': {'code': self.code_aul, 'conc': self.conc_aul},
            'boul': {'code': self.code_boul, 'conc': self.conc_boul},
            'gram': {'code': self.code_gram, 'conc': self.conc_gram},
            'oliv': {'code': self.code_oliv, 'conc': self.conc_oliv}
        }
        
        summary = {}
        for code_taxon, data in pollens.items():
            indice = data['code']
            concentration = data['conc']
            summary[code_taxon] = {
                'code': indice,
                'concentration': concentration,
                'qualificatif': INDICES_POLLENS.get(int(indice), "Inconnu"),
                'espece': CODE_TAXON.get(code_taxon, code_taxon),
                'couleur': self.get_color_by_level(int(indice)),
                'emoji': self.get_emoji_by_level(int(indice), emoji_style),
                'emoji_round': self.get_emoji_by_level(int(indice), "round"),
                'emoji_square': self.get_emoji_by_level(int(indice), "square")
            }
        
        return summary
    
    def get_concentrations(self) -> Dict[str, float]:
        """Retourne les concentrations de tous les pollens en grains/m³"""
        return {
            'ambr': self.conc_ambr,
            'arm': self.conc_arm,
            'aul': self.conc_aul,
            'boul': self.conc_boul,
            'gram': self.conc_gram,
            'oliv': self.conc_oliv
        }
    
    def get_highest_concentration(self) -> Tuple[str, float]:
        """Retourne le pollen avec la plus haute concentration"""
        concentrations = self.get_concentrations()
        highest = max(concentrations.items(), key=lambda x: x[1])
        return highest[0], highest[1]
    
    def get_dangerous_pollens(self) -> List[str]:
        """Retourne la liste des pollens à indice élevé (4+)"""
        pollens = {
            'ambr': self.code_ambr,
            'arm': self.code_arm,
            'aul': self.code_aul,
            'boul': self.code_boul,
            'gram': self.code_gram,
            'oliv': self.code_oliv
        }
        
        dangerous = []
        for code_taxon, indice in pollens.items():
            if indice >= 4:
                dangerous.append(CODE_TAXON.get(code_taxon, code_taxon))
        
        return dangerous
    
    def get_responsible_pollens(self) -> List[str]:
        """Retourne le ou les taxons responsables de l'indice selon l'API"""
        if not self.pollen_resp:
            return []
        
        # Le champ peut contenir plusieurs taxons séparés par des virgules ou des espaces
        # Exemples: "graminées,armoise", "GRAMINEE", "ARTEMISI GRAMINEE"
        pollen_resp_clean = self.pollen_resp.lower().strip()
        
        # Séparer par virgules ou espaces
        taxons_raw = []
        if ',' in pollen_resp_clean:
            taxons_raw = [t.strip() for t in pollen_resp_clean.split(',')]
        else:
            # Essayer de détecter plusieurs mots (séparés par des espaces)
            words = pollen_resp_clean.split()
            if len(words) > 1:
                taxons_raw = words
            else:
                taxons_raw = [pollen_resp_clean]
        
        taxons_formatted = []
        
        for taxon_raw in taxons_raw:
            if not taxon_raw:  # Skip empty strings
                continue
                
            # Chercher d'abord dans le mapping
            taxon_code = TAXON_MAPPING.get(taxon_raw)
            if taxon_code:
                # Utiliser le nom complet du CODE_TAXON
                taxon_name = CODE_TAXON.get(taxon_code, taxon_raw.title())
                if taxon_name not in taxons_formatted:  # Éviter les doublons
                    taxons_formatted.append(taxon_name)
            else:
                # Si pas trouvé, formater simplement la première lettre en majuscule
                formatted_name = taxon_raw.title()
                if formatted_name not in taxons_formatted:  # Éviter les doublons
                    taxons_formatted.append(formatted_name)
        
        return taxons_formatted
    
    def __str__(self) -> str:
        highest_pollen, highest_code = self.get_highest_pollen()
        espece = CODE_TAXON.get(highest_pollen, highest_pollen)
        qualif = INDICES_POLLENS.get(highest_code, "Inconnu")
        return f"Pollen - Plus élevé: {espece} ({qualif})"


class AtmoDataCollection:
    """Classe pour gérer une collection de données Atmo"""
    
    def __init__(self, data: Dict[str, Any], data_type: str):
        self.raw_data = data
        self.data_type = data_type
        self.features = data.get('features', [])
        
        # Créer les objets typés
        self.items = self._create_typed_objects()
    
    def _create_typed_objects(self) -> List[AtmoDataBase]:
        """Crée les objets typés selon le type de données"""
        objects = []
        
        for feature in self.features:
            if self.data_type == 'indices':
                objects.append(IndiceAtmo(feature))
            elif self.data_type == 'episodes':
                objects.append(EpisodePollution(feature))
            elif self.data_type == 'emissions':
                objects.append(EmissionData(feature))
            elif self.data_type == 'pollens':
                objects.append(IndicePollen(feature))
            else:
                objects.append(AtmoDataBase(feature))
        
        return objects
    
    def __len__(self) -> int:
        return len(self.items)
    
    def __iter__(self):
        return iter(self.items)
    
    def __getitem__(self, index):
        return self.items[index]
    
    def filter_by_aasqa(self, aasqa_code: str) -> 'AtmoDataCollection':
        """Filtre par code AASQA"""
        filtered_features = [
            item.raw_data for item in self.items 
            if item.aasqa == aasqa_code
        ]
        filtered_data = {
            'type': self.raw_data.get('type'),
            'features': filtered_features
        }
        return AtmoDataCollection(filtered_data, self.data_type)
    
    def filter_by_coordinates(self, center: Coordinates, radius_km: float) -> 'AtmoDataCollection':
        """Filtre par proximité géographique"""
        filtered_items = []
        for item in self.items:
            if item.has_coordinates():
                distance = item.coordinates.distance_to(center)
                if distance <= radius_km:
                    filtered_items.append(item.raw_data)
        
        filtered_data = {
            'type': self.raw_data.get('type'),
            'features': filtered_items
        }
        return AtmoDataCollection(filtered_data, self.data_type)
    
    def get_statistics(self) -> Dict[str, Any]:
        """Retourne des statistiques sur la collection"""
        stats = {
            'total_count': len(self.items),
            'aasqa_count': len(set(item.aasqa for item in self.items)),
            'data_type': self.data_type
        }
        
        # Statistiques spécifiques par type
        if self.data_type == 'indices' and self.items:
            indices_codes = [item.code_qual for item in self.items if hasattr(item, 'code_qual')]
            if indices_codes:
                stats['quality_stats'] = {
                    'moyenne': statistics.mean(indices_codes),
                    'min': min(indices_codes),
                    'max': max(indices_codes),
                    'bon_pourcentage': (sum(1 for x in indices_codes if x <= 2) / len(indices_codes)) * 100
                }
        
        elif self.data_type == 'episodes' and self.items:
            alerts_count = sum(1 for item in self.items if hasattr(item, 'is_alert_active') and item.is_alert_active())
            stats['alerts_active'] = alerts_count
            stats['alerts_percentage'] = (alerts_count / len(self.items)) * 100
        
        elif self.data_type == 'pollens' and self.items:
            alerts_count = sum(1 for item in self.items if hasattr(item, 'alerte') and item.alerte)
            stats['pollen_alerts'] = alerts_count
        
        return stats
    
    def to_summary(self) -> str:
        """Retourne un résumé textuel de la collection"""
        stats = self.get_statistics()
        summary = f"Collection {self.data_type.title()}: {stats['total_count']} éléments"
        
        if self.data_type == 'indices' and 'quality_stats' in stats:
            qs = stats['quality_stats']
            summary += f" - Qualité moyenne: {qs['moyenne']:.1f}, Bonne qualité: {qs['bon_pourcentage']:.1f}%"
        
        elif self.data_type == 'episodes' and 'alerts_active' in stats:
            summary += f" - Alertes actives: {stats['alerts_active']} ({stats['alerts_percentage']:.1f}%)"
        
        elif self.data_type == 'pollens' and 'pollen_alerts' in stats:
            summary += f" - Alertes pollen: {stats['pollen_alerts']}"
        
        return summary
