"""
Fonctions utilitaires pour le wrapper Atmo Data
"""

from typing import Optional, List, Dict, Any
from .constants import AASQA_CODES, ATMO_LICENCE_COURTE, ATMO_LICENCE_LONGUE, ATMO_LICENCE_COMPLETE


def get_aasqa_by_department(department: str) -> Optional[str]:
    """
    Trouve le code AASQA correspondant à un département
    
    Args:
        department: Code département (ex: '54', '75', '2A')
        
    Returns:
        str: Code AASQA ou None si non trouvé
    """
    for aasqa_code, aasqa_data in AASQA_CODES.items():
        if department in aasqa_data['departements']:
            return aasqa_code
    return None


def get_aasqa_info(aasqa_code: str) -> Optional[Dict[str, Any]]:
    """
    Récupère les informations complètes d'une AASQA
    
    Args:
        aasqa_code: Code AASQA (ex: '44')
        
    Returns:
        dict: Informations complètes ou None si non trouvé
    """
    return AASQA_CODES.get(aasqa_code)


def get_aasqa_website(aasqa_code: str) -> Optional[str]:
    """
    Récupère le site web d'une AASQA
    
    Args:
        aasqa_code: Code AASQA (ex: '44')
        
    Returns:
        str: URL du site web ou None si non trouvé
    """
    aasqa_info = AASQA_CODES.get(aasqa_code)
    return aasqa_info['site_web'] if aasqa_info else None


def list_departments_by_aasqa(aasqa_code: str) -> List[str]:
    """
    Liste les départements couverts par une AASQA
    
    Args:
        aasqa_code: Code AASQA (ex: '44')
        
    Returns:
        list: Liste des codes département
    """
    aasqa_info = AASQA_CODES.get(aasqa_code)
    return aasqa_info['departements'] if aasqa_info else []


def search_aasqa_by_name(search_term: str) -> List[Dict[str, Any]]:
    """
    Recherche des AASQA par nom d'organisme ou de région
    
    Args:
        search_term: Terme de recherche (case insensitive)
        
    Returns:
        list: Liste des AASQA correspondantes avec leurs informations
    """
    results = []
    search_lower = search_term.lower()
    
    for aasqa_code, aasqa_data in AASQA_CODES.items():
        if (search_lower in aasqa_data['organisme'].lower() or 
            search_lower in aasqa_data['region'].lower()):
            results.append({
                'code': aasqa_code,
                **aasqa_data
            })
    
    return results


def get_departments_count() -> Dict[str, int]:
    """
    Retourne le nombre de départements par AASQA
    
    Returns:
        dict: Code AASQA -> nombre de départements
    """
    return {
        aasqa_code: len(aasqa_data['departements']) 
        for aasqa_code, aasqa_data in AASQA_CODES.items()
    }


def validate_department_coverage() -> Dict[str, Any]:
    """
    Valide la couverture départementale et détecte les anomalies
    
    Returns:
        dict: Rapport de validation avec statistiques et anomalies
    """
    all_departments = []
    for aasqa_data in AASQA_CODES.values():
        all_departments.extend(aasqa_data['departements'])
    
    unique_departments = set(all_departments)
    duplicates = []
    seen = set()
    
    for dept in all_departments:
        if dept in seen:
            duplicates.append(dept)
        seen.add(dept)
    
    return {
        'total_departments': len(all_departments),
        'unique_departments': len(unique_departments),
        'duplicates': list(set(duplicates)),
        'has_duplicates': len(duplicates) > 0,
        'coverage_complete': len(unique_departments) == len(all_departments)
    }


def get_aasqa_statistics() -> Dict[str, Any]:
    """
    Génère des statistiques sur les AASQA
    
    Returns:
        dict: Statistiques détaillées
    """
    dept_counts = get_departments_count()
    validation = validate_department_coverage()
    
    max_coverage = max(dept_counts.values())
    min_coverage = min(dept_counts.values())
    
    max_aasqa = [code for code, count in dept_counts.items() if count == max_coverage]
    min_aasqa = [code for code, count in dept_counts.items() if count == min_coverage]
    
    return {
        'total_aasqa': len(AASQA_CODES),
        'total_departments_covered': validation['total_departments'],
        'unique_departments': validation['unique_departments'],
        'max_coverage': {
            'count': max_coverage,
            'aasqa_codes': max_aasqa,
            'aasqa_names': [AASQA_CODES[code]['organisme'] for code in max_aasqa]
        },
        'min_coverage': {
            'count': min_coverage,
            'aasqa_codes': min_aasqa,
            'aasqa_names': [AASQA_CODES[code]['organisme'] for code in min_aasqa]
        },
        'average_coverage': sum(dept_counts.values()) / len(dept_counts),
        'has_anomalies': validation['has_duplicates']
    }


def get_atmo_licence(format: str = "courte") -> str:
    """
    Retourne la mention légale Atmo France selon le format demandé
    
    Args:
        format: Format de la licence ("courte", "longue", "complete")
        
    Returns:
        str: Mention légale correspondante
        
    Examples:
        >>> print(get_atmo_licence("courte"))
        Atmo France / AASQA
        
        >>> print(get_atmo_licence("longue"))
        Atmo France et les Associations agréées de surveillance de la qualité de l'air
        
        >>> print(get_atmo_licence("complete"))
        Données sous licence ODbL (Open Database License)
        Source: Atmo France et les Associations agréées de surveillance de la qualité de l'air
        ...
    """
    format_lower = format.lower()
    
    if format_lower == "courte":
        return ATMO_LICENCE_COURTE
    elif format_lower == "longue":
        return ATMO_LICENCE_LONGUE
    elif format_lower == "complete":
        return ATMO_LICENCE_COMPLETE
    else:
        # Format par défaut si non reconnu
        return ATMO_LICENCE_COURTE


def print_atmo_licence(format: str = "courte") -> None:
    """
    Affiche la mention légale Atmo France selon le format demandé
    
    Args:
        format: Format de la licence ("courte", "longue", "complete")
        
    Examples:
        >>> print_atmo_licence("courte")
        Atmo France / AASQA
        
        >>> print_atmo_licence("complete")
        Données sous licence ODbL (Open Database License)
        Source: Atmo France et les Associations agréées de surveillance de la qualité de l'air
        ...
    """
    print(get_atmo_licence(format))