#!/usr/bin/env python3
"""
Test simple du client avec les nouveaux objets typés
"""

from atmo_data_wrapper import AtmoDataClient
from atmo_data_wrapper import AtmoDataCollection, IndiceAtmo

def test_client_integration():
    """Test que le client retourne bien les bons types d'objets"""
    print("=== Test d'intégration client/modèles ===\n")
    
    client = AtmoDataClient()
    
    # Test avec données simulées (sans vraie connexion API)
    
    # Simuler _make_request pour retourner des données de test
    def mock_make_request(endpoint, params=None):
        return {
            "type": "FeatureCollection",
            "features": [
                {
                    "type": "Feature",
                    "geometry": {
                        "type": "Point",
                        "coordinates": [2.3522, 48.8566]
                    },
                    "properties": {
                        "aasqa": "11",
                        "code_qual": 3,
                        "lib_qual": "Dégradé",
                        "lib_zone": "Test Zone"
                    }
                }
            ]
        }
    
    # Remplacer temporairement la méthode
    original_make_request = client._make_request
    client._make_request = mock_make_request
    client.token = "test_token"  # Simuler une connexion
    
    try:
        # Test 1: get_indices_atmo avec format geojson -> AtmoDataCollection
        print("1. Test get_indices_atmo format geojson...")
        result = client.get_indices_atmo(format="geojson")
        
        if isinstance(result, AtmoDataCollection):
            print("✅ Retourne bien AtmoDataCollection")
            print(f"   Type de données: {result.data_type}")
            print(f"   Nombre d'éléments: {len(result)}")
            
            # Vérifier que les éléments sont du bon type
            if len(result) > 0 and isinstance(result[0], IndiceAtmo):
                print("✅ Les éléments sont bien des objets IndiceAtmo")
                print(f"   Zone: {result[0].lib_zone}")
                print(f"   Qualité: {result[0].get_qualificatif()}")
            else:
                print("❌ Les éléments ne sont pas des objets IndiceAtmo")
        else:
            print(f"❌ Retourne {type(result)} au lieu de AtmoDataCollection")
        
        print()
        
        # Test 2: Simuler format CSV -> dict
        def mock_csv_request(endpoint, params=None):
            return {"data": "csv,data,here"}
        
        client._make_request = mock_csv_request
        
        print("2. Test get_indices_atmo format csv...")
        result_csv = client.get_indices_atmo(format="csv")
        
        if isinstance(result_csv, dict):
            print("✅ Format CSV retourne bien un dict")
        else:
            print(f"❌ Format CSV retourne {type(result_csv)} au lieu de dict")
        
        print()
        
        # Test 3: Test différents endpoints
        client._make_request = mock_make_request
        
        endpoints_to_test = [
            ("get_episodes_3jours", "episodes"),
            ("get_emissions", "emissions"),
            ("get_indices_pollens", "pollens")
        ]
        
        print("3. Test autres endpoints...")
        for method_name, expected_type in endpoints_to_test:
            method = getattr(client, method_name)
            result = method(format="geojson")
            
            if isinstance(result, AtmoDataCollection) and result.data_type == expected_type:
                print(f"✅ {method_name} retourne bien AtmoDataCollection({expected_type})")
            else:
                print(f"❌ {method_name} problème de type: {type(result)}, data_type: {getattr(result, 'data_type', 'N/A')}")
        
        print("\n=== Test terminé ===")
        
    finally:
        # Restaurer la méthode originale
        client._make_request = original_make_request

if __name__ == "__main__":
    test_client_integration()